#!/usr/bin/ruby
##################################################################################
# Name : LocateR
# Version : 2.00.A
# Created On : Wed Sep 27 2006
# Last Modified On : 
# Author : Binny V A
# Contact : binnyva[AT]gmail[DOT]com
# Webpage : 
# This is a small GUI for the 'locate' command on Linux system. This will 
#	search the computer using the locate command and will show the results in that 
#	window itself. Using the locate command is much quicker than searching for a 
#	file in the traditional way(find)
##################################################################################

require 'tk'

$case_sensetive	= TkVariable.new(false)
$regexp			= TkVariable.new(false)
$filename_only	= TkVariable.new(false)
$results 		= Array.new()

# A small debug message will be shown in the textarea
def show(text)
	if ($verbose.value == '1') then
		$txt.insert('end',text+"\n") 
		Tk.update()
	end
end

def filter(str)
	ignore_case = true
	ignore_case = false if $case_sensetive.value == '1'

	reg = Regexp.new(str,ignore_case)
	$results = $results.grep(reg)

	$res.delete(0,'end') #Deleting existing results
	$results.each { |file|
		$res.insert('end',file)
	}
end

def regExpWarn(from)
	warning = ''
	warning = "'" + from + "' uses Regular Expression Searching. " if (from != '')

	warning += "RegExp searches take more time than normal searches." 
	$status_bar.configure('text'=>$warning)
end

def search()
	return if($search.value == '')

	$status_bar.configure('text'=>'Searching...')
	$res.delete(0,'end') #Deleting existing results
	Tk.update()
	
	include_search_text = true
	#Create the command based on user settings
	command = "locate "
	command += "-i " if $case_sensetive.value == '1'
	
	if ($regexp.value == '1') then
		command += "-r "
	end
	
	command += "-n 100 " #A safety measure.

	command += $search.value if include_search_text

	#Execute the command with ` to make sure that we get the results back in the program.
	command_result = eval("`"+command+"`")
	
	$results = command_result.split("\n")

	#Filter the results where the keyword does not appear in the filename
	filter('/[^/]*' + $search.value + '[^/]*$') if ($filename_only.value == '1')
	filter('^' + $search_folder.value)  if ($search_folder.value != "")
	
	#Insert each result into the list
	$results.each { |file|
		$res.insert('end',file)
	}

	$status_bar.configure('text'=>'Done')
	
end

def openInEditor(file)
	exec("kwrite",file)
end

def openFolder(folder)
	exec("konqueror",folder)
end

def rightClick(e)
	index = $res.nearest(e.y)
	$res.selection_clear(0,'end')
	$res.selection_set(index)
	
	file = $res.get(index)
	folder = File.dirname(file)
	
	$mnu.delete(0,'end')
	$mnu.add_command('label'=>"Open In Editor",'command'=>proc{openInEditor(file)})
	$mnu.add_command('label'=>"Open Folder",'command'=>proc{openFolder(folder)})

	$mnu.post(e.x+290, e.y+165)
end

def alert(msg)
	Tk.messageBox('message'=>msg)
end

################################## GUI Building ##################################
root = TkRoot.new { title "LocateR" }

### Search For
$search = TkVariable.new("")
frm_search = TkFrame.new(root).pack('anchor'=>'w','fill'=>'x','expand'=>1)
TkLabel.new(frm_search,'text'=>"Search",'width'=>20,'anchor'=>'w').pack('side'=>'left')
TkEntry.new(frm_search,'textvariable'=>$search,'width'=>50,'bg'=>'white').pack('fill'=>'x','expand'=>1)

### In Folder
$search_folder = TkVariable.new(Dir.pwd)
frm_search_folder = TkFrame.new(root).pack('anchor'=>'w','fill'=>'x','expand'=>1)
TkLabel.new(frm_search_folder,'text'=>"Search in Folder",'width'=>20,'anchor'=>'w').pack('side'=>'left')
TkEntry.new(frm_search_folder,'textvariable'=>$search_folder,'width'=>40,'bg'=>'white').pack('fill'=>'x','expand'=>1,'side'=>'left')
TkButton.new(frm_search_folder) {
	text	"Browse..."
	anchor	'e'
	command	proc { 
		folder = Tk.chooseDirectory('initialdir'=>$search_folder.value)
		if(folder != '') then
			$search_folder.value = folder
		end
	}
}.pack('side'=>'left')


### Settings
frm_setting = TkFrame.new(root).pack('anchor'=>'w')
TkCheckbutton.new(frm_setting,'text'=>"Case Sensitive",'variable'=>$case_sensetive).pack('side'=>'left')
TkCheckbutton.new(frm_setting) {
	text		"RegExp"
	variable	$regexp
	command		proc { regExpWarn('') }
}.pack('side'=>'left')

#Search only the filename part of the path - uses RegExp.
TkCheckbutton.new(frm_setting) {
	text		"Search Filename Only"
	variable	$filename_only
	command		proc { regExpWarn('Search Filename Only') }
}.pack('side'=>'left')


#Show the search button
TkButton.new(root) {
	text "Search"
	command proc {
		search()
	}
}.pack()

### Results display
TkLabel.new(root,'text'=>"Results...").pack('anchor'=>'w');
frm_results = TkFrame.new(root).pack('fill'=>'both', 'expand'=>1)

$res = TkListbox.new(frm_results,'bg'=>'white','selectmode'=>'single').pack('side'=>'left', 'fill'=>'both', 'expand'=>1);
srl_y_res = TkScrollbar.new(frm_results).pack('fill'=>'y','expand'=>1)
srl_x_res = TkScrollbar.new(root).pack('fill'=>'x','expand'=>1)

$res.yscrollbar(srl_y_res);
$res.xscrollbar(srl_x_res);
$res.bind('ButtonPress-3',proc{|e| rightClick(e)})
$res.bind('Double-ButtonPress-1',proc{openInEditor($res.get($res.curselection()[0]))})

### The filter thingy
frm_filter = TkFrame.new(root).pack('anchor'=>'w');
TkLabel.new(frm_filter,'text'=>"Filter").pack('side'=>'left');
ent_filter = TkEntry.new(frm_filter,'validate'=>"key",'validatecommand'=>proc {|s|
		filter(s.value)
		return true
	},'width'=>50,'bg'=>'white').pack('side'=>'left');

#The Menu that comes on right click
$mnu = TkMenu.new('tearoff'=>0,'parent'=>$res)


### The Status bar
$status_bar = TkLabel.new(root,'text'=>' ').pack('anchor'=>'w')

root.bind('Key-Escape',proc{exit})
root.bind('ButtonPress-1',proc{$mnu.unpost()})
Tk.mainloop

########################### TO DO ################################
